package i5d.gui;
import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Graphics;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.util.Hashtable;

import i5d.Image5D;
import ij.IJ;
import ij.gui.*;
/*
 * Created on 29.05.2005
 */
import ij.plugin.frame.RoiManager;

/** Canvas compatible with Image5DLayout.
 * @author Joachim Walter
 */
public class Image5DCanvas extends ImageCanvas {

    /**
	 * 
	 */
	private static final long serialVersionUID = -8563611296852137396L;

	/**
     * @param imp
     */
    public Image5DCanvas(Image5D imp) {
        super(imp);
    }
    
    public Image5D getImage() {    	
    	return (Image5D)imp;
    }

    
	/** Adjust the canvas size symmetrically about the middle
     *  of the srcRect, if the user resizes the window. 
	 * 	Called from Image5DLayout.layoutContainer().
     * */
	protected Dimension resizeCanvasI5D(int width, int height) {
	    double magnification = getMagnification();        
     
		if ( width>imageWidth*magnification )
			width = (int)(imageWidth*magnification);
		if ( height>imageHeight*magnification )
			height = (int)(imageHeight*magnification);
		setDrawingSize(width, height);
		Dimension dim = new Dimension(width, height);

        int newSrcRectWidth = (int)(width/magnification);
        // Prevent display of zoomIndicator due to rounding error.
        if (Math.round(magnification) != magnification 
                && (int)((width+1)/magnification) >= imageWidth) {
            newSrcRectWidth = imageWidth;
        }
        int newSrcRectHeight = (int)(height/magnification);
        // Prevent display of zoomIndicator due to rounding error.
        if (Math.round(magnification) != magnification 
                && (int)((height+1)/magnification) >= imageHeight) {
            newSrcRectHeight = imageHeight;
        }
        
        srcRect.x = srcRect.x + (srcRect.width - newSrcRectWidth )/2;
        if (srcRect.x < 0)
            srcRect.x = 0;
        srcRect.y = srcRect.y + (srcRect.height - newSrcRectHeight )/2;
        if (srcRect.y < 0)
            srcRect.y = 0;
        srcRect.width = newSrcRectWidth;
        srcRect.height = newSrcRectHeight;
		if ((srcRect.x+srcRect.width)>imageWidth)
			srcRect.x = imageWidth-srcRect.width;
		if ((srcRect.y+srcRect.height)>imageHeight)
			srcRect.y = imageHeight-srcRect.height;
		repaint();
		
		return dim;
	}
    
    protected boolean canEnlarge(int newWidth, int newHeight) {
        if ((flags&Event.SHIFT_MASK)!=0 || IJ.shiftKeyDown())
            return false;
        ImageWindow win = imp.getWindow();
        if (win==null) return false;
        Rectangle r1 = win.getBounds();
        r1.width = newWidth + 20;
        r1.height = newHeight + 50;
        if (imp.getStackSize()>1)
            r1.height += 20;
        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
        boolean fitsOnScreen = r1.x+r1.width<screen.width && r1.y+r1.height+30<screen.height;
        return fitsOnScreen;
    }

    /* copied and modified from ImageCanvas */
    public void zoomOut(int x, int y) {
        if (magnification<=0.03125)
            return;
        double newMag = getLowerZoomLevel(magnification);
        //if (newMag==imp.getWindow().getInitialMagnification()) {
        //  unzoom();
        //  return;
        //}
        if (imageWidth*newMag>dstWidth) {
            int w = (int)Math.round(dstWidth/newMag);
            if (w*newMag<dstWidth) w++;
            int h = (int)Math.round(dstHeight/newMag);
            if (h*newMag<dstHeight) h++;
            x = offScreenX(x);
            y = offScreenY(y);
            Rectangle r = new Rectangle(x-w/2, y-h/2, w, h);
            if (r.x<0) r.x = 0;
            if (r.y<0) r.y = 0;
            if (r.x+w>imageWidth) r.x = imageWidth-w;
            if (r.y+h>imageHeight) r.y = imageHeight-h;
            srcRect = r;
            setMagnification(newMag);
        }
        else {
            srcRect = new Rectangle(0, 0, imageWidth, imageHeight);
            setDrawingSize((int)(imageWidth*newMag), (int)(imageHeight*newMag));
            //setDrawingSize(dstWidth/2, dstHeight/2);
            setMagnification(newMag);
            imp.getWindow().pack();
        }
        //IJ.write(newMag + " " + srcRect.x+" "+srcRect.y+" "+srcRect.width+" "+srcRect.height+" "+dstWidth + " " + dstHeight);
        //IJ.write(srcRect.x + " " + srcRect.width + " " + dstWidth);
        repaint();
    }

    /* copied and modified from ImageCanvas */
    public void zoomIn(int x, int y) {
        if (magnification>=32)
            return;
        double newMag = getHigherZoomLevel(magnification);
        int newWidth = (int)(imageWidth*newMag);  
        int newHeight = (int)(imageHeight*newMag);
        if (canEnlarge(newWidth, newHeight)) {
            setDrawingSize(newWidth, newHeight);
            setMagnification(newMag);
            imp.getWindow().pack();
        } else {
            int w = (int)Math.round(dstWidth/newMag);
            if (w*newMag<dstWidth) w++;
            int h = (int)Math.round(dstHeight/newMag);
            if (h*newMag<dstHeight) h++;
            x = offScreenX(x);
            y = offScreenY(y);
            Rectangle r = new Rectangle(x-w/2, y-h/2, w, h);
            if (r.x<0) r.x = 0;
            if (r.y<0) r.y = 0;
            if (r.x+w>imageWidth) r.x = imageWidth-w;
            if (r.y+h>imageHeight) r.y = imageHeight-h;
            srcRect = r;
            setMagnification(newMag);
        }
//        setMagnification(newMag);
        repaint();
    }
      
}
